@@ -1,11 +1,6 @@
 #ifndef SRC_COMMON_XLAT_H_
 #define SRC_COMMON_XLAT_H_
 
-#include <linux/types.h>
-#ifndef __KERNEL__
-	#include <stdbool.h>
-#endif
-
 #define JOOL_LICENSE "GPL v2"
 
 /**
@@ -13,8 +8,8 @@
  * relocate them, please make sure to also update dkms.conf accordingly.
  */
 #define JOOL_VERSION_MAJOR 4
-#define JOOL_VERSION_MINOR 0
-#define JOOL_VERSION_REV 7
+#define JOOL_VERSION_MINOR 1
+#define JOOL_VERSION_REV 2
 #define JOOL_VERSION_DEV 0
 
 /** See http://stackoverflow.com/questions/195975 */
