@@ -33,7 +33,8 @@ struct addrxlat_result {
 };
 
 struct addrxlat_result addrxlat_siit64(struct xlator *instance,
-		struct in6_addr *in, struct result_addrxlat64 *out);
+		struct in6_addr *in, struct result_addrxlat64 *out,
+		bool enable_blacklists);
 struct addrxlat_result addrxlat_siit46(struct xlator *instance,
 		__be32 in, struct result_addrxlat46 *out,
 		bool enable_eam, bool enable_blacklists);
