@@ -1,12 +1,13 @@
-#include "handling_hairpinning_siit.h"
+#include "mod/common/steps/handling_hairpinning_siit.h"
 
 #include "mod/common/log.h"
 #include "mod/common/rfc7915/core.h"
+#include "mod/common/steps/compute_outgoing_tuple.h"
 #include "mod/common/steps/send_packet.h"
 
 bool is_hairpin_siit(struct xlation *state)
 {
-	return pkt_is_intrinsic_hairpin(&state->out);
+	return state->is_hairpin;
 }
 
 verdict handling_hairpinning_siit(struct xlation *old)
@@ -14,10 +15,11 @@ verdict handling_hairpinning_siit(struct xlation *old)
 	struct xlation new;
 	verdict result;
 
-	log_debug("Packet is a hairpin. U-turning...");
+	log_debug(old, "Packet is hairpinning. U-turning...");
 
 	new.jool = old->jool;
 	new.in = old->out;
+	new.is_hairpin = true;
 
 	result = translating_the_packet(&new);
 	if (result != VERDICT_CONTINUE)
@@ -26,6 +28,6 @@ verdict handling_hairpinning_siit(struct xlation *old)
 	if (result != VERDICT_CONTINUE)
 		return result;
 
-	log_debug("Done hairpinning.");
+	log_debug(old, "Done hairpinning.");
 	return VERDICT_CONTINUE;
 }
